@@ -25,7 +25,6 @@ import uuid
 
 import croniter
 from oslo_config import cfg
-from oslo_context import context
 from oslo_log import log
 from oslo_utils import netutils
 from oslo_utils import timeutils
@@ -49,19 +48,16 @@ from aodh import keystone_client
 from aodh import messaging
 from aodh import notifier
 from aodh.storage import models
-from aodh import utils
 
 LOG = log.getLogger(__name__)
 
 
 ALARM_API_OPTS = [
     cfg.IntOpt('user_alarm_quota',
-               default=None,
                deprecated_group="alarm",
                help='Maximum number of alarms defined for a user.'
                ),
     cfg.IntOpt('project_alarm_quota',
-               default=None,
                deprecated_group="alarm",
                help='Maximum number of alarms defined for a project.'
                ),
@@ -375,7 +371,8 @@ class Alarm(base.Base):
         for k in d:
             if k.endswith('_rule'):
                 del d[k]
-        d['rule'] = getattr(self, "%s_rule" % self.type).as_dict()
+        rule = getattr(self, "%s_rule" % self.type)
+        d['rule'] = rule if isinstance(rule, dict) else rule.as_dict()
         if self.time_constraints:
             d['time_constraints'] = [tc.as_dict()
                                      for tc in self.time_constraints]
@@ -491,7 +488,14 @@ def _send_notification(event, payload):
     notifier = messaging.get_notifier(transport, publisher_id="aodh.api")
     # FIXME(sileht): perhaps we need to copy some infos from the
     # pecan request headers like nova does
-    notifier.info(context.RequestContext(), notification, payload)
+    notifier.info({}, notification, payload)
+
+
+def stringify_timestamps(data):
+    """Stringify any datetimes in given dict."""
+    return dict((k, v.isoformat()
+                 if isinstance(v, datetime.datetime) else v)
+                for (k, v) in six.iteritems(data))
 
 
 class AlarmController(rest.RestController):
@@ -530,7 +534,7 @@ class AlarmController(rest.RestController):
         if not data:
             return
         type = type or models.AlarmChange.RULE_CHANGE
-        scrubbed_data = utils.stringify_timestamps(data)
+        scrubbed_data = stringify_timestamps(data)
         detail = json.dumps(scrubbed_data)
         user_id = pecan.request.headers.get('X-User-Id')
         project_id = pecan.request.headers.get('X-Project-Id')
@@ -695,7 +699,7 @@ class AlarmsController(rest.RestController):
         if not pecan.request.cfg.record_history:
             return
         type = models.AlarmChange.CREATION
-        scrubbed_data = utils.stringify_timestamps(data)
+        scrubbed_data = stringify_timestamps(data)
         detail = json.dumps(scrubbed_data)
         user_id = pecan.request.headers.get('X-User-Id')
         project_id = pecan.request.headers.get('X-Project-Id')
