@@ -30,9 +30,10 @@ import uuid
 
 import aodh
 from aodh import coordination
-from aodh.i18n import _, _LW
+from aodh.i18n import _
 from aodh import keystone_client
 from aodh import messaging
+from aodh import queue
 from aodh import rpc
 from aodh import storage
 from aodh.storage import models
@@ -57,9 +58,12 @@ OPTS = [
 class Evaluator(object):
     """Base class for alarm rule evaluator plugins."""
 
-    def __init__(self, conf, notifier):
+    def __init__(self, conf):
         self.conf = conf
-        self.notifier = notifier
+        if conf.ipc_protocol == 'rpc':
+            self.notifier = rpc.RPCAlarmNotifier(self.conf)
+        else:
+            self.notifier = queue.AlarmNotifier(self.conf)
         self.storage_conn = None
         self._ks_client = None
         self._alarm_change_notifier = None
@@ -105,28 +109,20 @@ class Evaluator(object):
         self._alarm_change_notifier.info({},
                                          notification, payload)
 
-    def _refresh(self, alarm, state, reason, reason_data):
+    def _refresh(self, alarm, state, reason, reason_data, always_record=False):
         """Refresh alarm state."""
         try:
             previous = alarm.state
             alarm.state = state
-            if previous != state:
+            if previous != state or always_record:
                 LOG.info(_('alarm %(id)s transitioning to %(state)s because '
                            '%(reason)s') % {'id': alarm.alarm_id,
                                             'state': state,
                                             'reason': reason})
-                try:
-                    self._storage_conn.update_alarm(alarm)
-                except storage.AlarmNotFound:
-                    LOG.warning(_LW("Skip updating this alarm's state, the"
-                                    "alarm: %s has been deleted"),
-                                alarm.alarm_id)
-                else:
-                    self._record_change(alarm)
-                if self.notifier:
-                    self.notifier.notify(alarm, previous, reason, reason_data)
-            elif alarm.repeat_actions and self.notifier:
-                self.notifier.notify(alarm, previous, reason, reason_data)
+
+                self._storage_conn.update_alarm(alarm)
+                self._record_change(alarm)
+            self.notifier.notify(alarm, previous, reason, reason_data)
         except Exception:
             # retry will occur naturally on the next evaluation
             # cycle (unless alarm state reverts in the meantime)
@@ -180,16 +176,18 @@ class Evaluator(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AlarmService(object):
+class AlarmEvaluationService(os_service.Service):
+
+    PARTITIONING_GROUP_NAME = "alarm_evaluator"
     EVALUATOR_EXTENSIONS_NAMESPACE = "aodh.evaluator"
 
     def __init__(self, conf):
-        super(AlarmService, self).__init__()
+        super(AlarmEvaluationService, self).__init__()
         self.conf = conf
         self.storage_conn = None
         self._load_evaluators()
-        self.partition_coordinator = coordination.PartitionCoordinator(conf)
+        self.partition_coordinator = coordination.PartitionCoordinator(
+            conf.coordination.backend_url)
 
     @property
     def _storage_conn(self):
@@ -201,7 +199,7 @@ class AlarmService(object):
         self.evaluators = extension.ExtensionManager(
             namespace=self.EVALUATOR_EXTENSIONS_NAMESPACE,
             invoke_on_load=True,
-            invoke_args=(self.conf, rpc.RPCAlarmNotifier(self.conf),)
+            invoke_args=(self.conf,)
         )
 
     def _evaluate_assigned_alarms(self):
@@ -226,19 +224,6 @@ class AlarmService(object):
         except Exception:
             LOG.exception(_('Failed to evaluate alarm %s'), alarm.alarm_id)
 
-    @abc.abstractmethod
-    def _assigned_alarms(self):
-        pass
-
-
-class AlarmEvaluationService(AlarmService, os_service.Service):
-
-    PARTITIONING_GROUP_NAME = "alarm_evaluator"
-
-    def __init__(self, conf):
-        super(AlarmEvaluationService, self).__init__(conf)
-        self.partition_coordinator = coordination.PartitionCoordinator(conf)
-
     def start(self):
         super(AlarmEvaluationService, self).start()
         self.partition_coordinator.start()
@@ -267,8 +252,5 @@ class AlarmEvaluationService(AlarmService, os_service.Service):
         # those alarms.
         all_alarms = self._storage_conn.get_alarms(enabled=True,
                                                    exclude=dict(type='event'))
-        all_alarms = list(all_alarms)
-        all_alarm_ids = [a.alarm_id for a in all_alarms]
-        selected = self.partition_coordinator.extract_my_subset(
-            self.PARTITIONING_GROUP_NAME, all_alarm_ids)
-        return list(filter(lambda a: a.alarm_id in selected, all_alarms))
+        return self.partition_coordinator.extract_my_subset(
+            self.PARTITIONING_GROUP_NAME, all_alarms)
