@@ -37,13 +37,13 @@ class FunctionalTest(db_test_base.TestBase):
 
     def setUp(self):
         super(FunctionalTest, self).setUp()
-        conf = service.prepare_service([])
+        conf = service.prepare_service(argv=[], config_files=[])
         self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.setup_messaging(self.CONF)
 
         self.CONF.set_override('policy_file',
                                os.path.abspath('etc/aodh/policy.json'),
-                               group='oslo_policy')
+                               group='oslo_policy', enforce_type=True)
         self.app = self._make_app()
 
     def _make_app(self):
