@@ -20,17 +20,15 @@ import tooz.coordination
 from aodh import coordination
 from aodh import service
 from aodh.tests import base
-from aodh import utils
 
 
 class MockToozCoordinator(object):
     def __init__(self, member_id, shared_storage):
         self._member_id = member_id
         self._groups = shared_storage
-        self.is_started = False
 
     def start(self):
-        self.is_started = True
+        pass
 
     def stop(self):
         pass
@@ -106,11 +104,45 @@ class MockAsyncError(tooz.coordination.CoordAsyncResult):
         return True
 
 
+class TestHashRing(base.BaseTestCase):
+    def test_hash_ring(self):
+        num_nodes = 10
+        num_keys = 1000
+
+        nodes = [str(x) for x in range(num_nodes)]
+        hr = coordination.HashRing(nodes)
+
+        buckets = [0] * num_nodes
+        assignments = [-1] * num_keys
+        for k in range(num_keys):
+            n = int(hr.get_node(str(k)))
+            self.assertTrue(0 <= n <= num_nodes)
+            buckets[n] += 1
+            assignments[k] = n
+
+        # at least something in each bucket
+        self.assertTrue(all((c > 0 for c in buckets)))
+
+        # approximately even distribution
+        diff = max(buckets) - min(buckets)
+        self.assertTrue(diff < 0.3 * (num_keys / num_nodes))
+
+        # consistency
+        num_nodes += 1
+        nodes.append(str(num_nodes + 1))
+        hr = coordination.HashRing(nodes)
+        for k in range(num_keys):
+            n = int(hr.get_node(str(k)))
+            assignments[k] -= n
+        reassigned = len([c for c in assignments if c != 0])
+        self.assertTrue(reassigned < num_keys / num_nodes)
+
+
 class TestPartitioning(base.BaseTestCase):
 
     def setUp(self):
         super(TestPartitioning, self).setUp()
-        conf = service.prepare_service([])
+        conf = service.prepare_service(argv=[], config_files=[])
         self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.shared_storage = {}
 
@@ -118,11 +150,12 @@ class TestPartitioning(base.BaseTestCase):
                                      coordinator_cls=None):
         coordinator_cls = coordinator_cls or MockToozCoordinator
         self.CONF.set_override('backend_url', 'xxx://yyy',
-                               group='coordination')
+                               group='coordination', enforce_type=True)
         with mock.patch('tooz.coordination.get_coordinator',
                         lambda _, member_id:
                         coordinator_cls(member_id, shared_storage)):
-            pc = coordination.PartitionCoordinator(self.CONF, agent_id)
+            pc = coordination.PartitionCoordinator(
+                self.CONF.coordination.backend_url, agent_id)
             pc.start()
             return pc
 
@@ -164,7 +197,7 @@ class TestPartitioning(base.BaseTestCase):
         agents = ['agent_%s' % i for i in range(10)]
 
         expected_resources = [list() for _ in range(len(agents))]
-        hr = utils.HashRing(agents)
+        hr = coordination.HashRing(agents)
         for r in all_resources:
             key = agents.index(hr.get_node(r))
             expected_resources[key].append(r)
@@ -211,7 +244,7 @@ class TestPartitioning(base.BaseTestCase):
 
     def test_group_id_none(self):
         coord = self._get_new_started_coordinator({}, 'a')
-        self.assertTrue(coord._coordinator.is_started)
+        self.assertTrue(coord._started)
 
         with mock.patch.object(coord._coordinator, 'join_group') as mocked:
             coord.join_group(None)
@@ -222,8 +255,9 @@ class TestPartitioning(base.BaseTestCase):
 
     def test_stop(self):
         coord = self._get_new_started_coordinator({}, 'a')
-        self.assertTrue(coord._coordinator.is_started)
+        self.assertTrue(coord._started)
         coord.join_group("123")
         coord.stop()
         self.assertIsEmpty(coord._groups)
+        self.assertFalse(coord._started)
         self.assertIsNone(coord._coordinator)
