@@ -12,7 +12,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-"""Tests for aodh.service.SingletonAlarmService.
+"""Tests for aodh.evaluator.AlarmEvaluationService.
 """
 import mock
 from oslo_config import fixture as fixture_config
@@ -26,7 +26,7 @@ from aodh.tests import base as tests_base
 class TestAlarmEvaluationService(tests_base.BaseTestCase):
     def setUp(self):
         super(TestAlarmEvaluationService, self).setUp()
-        conf = service.prepare_service([])
+        conf = service.prepare_service(argv=[], config_files=[])
         self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.setup_messaging(self.CONF)
 
@@ -57,7 +57,8 @@ class TestAlarmEvaluationService(tests_base.BaseTestCase):
                                test_interval)
         self.CONF.set_override('heartbeat',
                                coordination_heartbeat,
-                               group='coordination')
+                               group='coordination',
+                               enforce_type=True)
         with mock.patch('aodh.storage.get_connection_from_config',
                         return_value=self.storage_conn):
             p_coord_mock = self.svc.partition_coordinator
@@ -94,7 +95,7 @@ class TestAlarmEvaluationService(tests_base.BaseTestCase):
                             coordination_heartbeat=5)
 
     def test_evaluation_cycle(self):
-        alarm = mock.Mock(type='threshold', alarm_id="alarm_id1")
+        alarm = mock.Mock(type='threshold')
         self.storage_conn.get_alarms.return_value = [alarm]
         with mock.patch('aodh.storage.get_connection_from_config',
                         return_value=self.storage_conn):
@@ -104,7 +105,7 @@ class TestAlarmEvaluationService(tests_base.BaseTestCase):
             self.svc._evaluate_assigned_alarms()
 
             p_coord_mock.extract_my_subset.assert_called_once_with(
-                self.svc.PARTITIONING_GROUP_NAME, ["alarm_id1"])
+                self.svc.PARTITIONING_GROUP_NAME, [alarm])
             self.threshold_eval.evaluate.assert_called_once_with(alarm)
 
     def test_evaluation_cycle_with_bad_alarm(self):
