@@ -1,5 +1,5 @@
 #
-# Copyright 2013-2014 eNovance
+# Copyright 2013-2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -15,7 +15,6 @@
 
 import mock
 from oslo_config import fixture as fixture_config
-from oslo_context import context
 from oslo_serialization import jsonutils
 from oslotest import mockpatch
 import requests
@@ -41,22 +40,42 @@ NOTIFICATION = dict(alarm_id='foobar',
                     current='ALARM')
 
 
+class TestAlarmNotifierService(tests_base.BaseTestCase):
+
+    def setUp(self):
+        super(TestAlarmNotifierService, self).setUp()
+        conf = service.prepare_service(argv=[], config_files=[])
+        self.CONF = self.useFixture(fixture_config.Config(conf)).conf
+        self.setup_messaging(self.CONF)
+
+    def test_init_host_rpc(self):
+        self.CONF.set_override('ipc_protocol', 'rpc')
+        self.service = notifier.AlarmNotifierService(self.CONF)
+        self.service.start()
+        self.service.stop()
+
+    def test_init_host_queue(self):
+        self.service = notifier.AlarmNotifierService(self.CONF)
+        self.service.start()
+        self.service.stop()
+
+
 class TestAlarmNotifier(tests_base.BaseTestCase):
 
     def setUp(self):
         super(TestAlarmNotifier, self).setUp()
-        conf = service.prepare_service([])
+        conf = service.prepare_service(argv=[], config_files=[])
         self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.setup_messaging(self.CONF)
+        self.zaqar = FakeZaqarClient(self)
+        self.useFixture(mockpatch.Patch(
+            'aodh.notifier.zaqar.ZaqarAlarmNotifier.get_zaqar_client',
+            return_value=self.zaqar))
         self.service = notifier.AlarmNotifierService(self.CONF)
         self.useFixture(mockpatch.Patch(
             'oslo_context.context.generate_request_id',
             self._fake_generate_request_id))
 
-    def test_init_host(self):
-        self.service.start()
-        self.service.stop()
-
     def test_notify_alarm(self):
         data = {
             'actions': ['test://'],
@@ -68,7 +87,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
             'reason': 'Everything is on fire',
             'reason_data': {'fire': 'everywhere'}
         }
-        self.service.notify_alarm(context.get_admin_context(), data)
+        self.service.notify_alarm({}, data)
         notifications = self.service.notifiers['test'].obj.notifications
         self.assertEqual(1, len(notifications))
         self.assertEqual((urlparse.urlsplit(data['actions'][0]),
@@ -82,19 +101,15 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                          notifications[0])
 
     def test_notify_alarm_no_action(self):
-        self.service.notify_alarm(context.get_admin_context(), {})
+        self.service.notify_alarm({}, {})
 
     def test_notify_alarm_log_action(self):
-        self.service.notify_alarm(context.get_admin_context(),
+        self.service.notify_alarm({},
                                   {
                                       'actions': ['log://'],
                                       'alarm_id': 'foobar',
                                       'condition': {'threshold': 42}})
 
-    @staticmethod
-    def _fake_spawn_n(func, *args, **kwargs):
-        func(*args, **kwargs)
-
     @staticmethod
     def _notification(action):
         notification = {}
@@ -111,15 +126,14 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
     def test_notify_alarm_rest_action_ok(self):
         action = 'http://host/action'
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                poster.assert_called_with(action, data=mock.ANY,
-                                          headers=mock.ANY)
-                args, kwargs = poster.call_args
-                self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            poster.assert_called_with(action, data=mock.ANY,
+                                      headers=mock.ANY)
+            args, kwargs = poster.call_args
+            self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_client_cert(self):
         action = 'https://host/action'
@@ -127,16 +141,15 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
 
         self.CONF.set_override("rest_notifier_certificate_file", certificate)
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                poster.assert_called_with(action, data=mock.ANY,
-                                          headers=mock.ANY,
-                                          cert=certificate, verify=True)
-                args, kwargs = poster.call_args
-                self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            poster.assert_called_with(action, data=mock.ANY,
+                                      headers=mock.ANY,
+                                      cert=certificate, verify=True)
+            args, kwargs = poster.call_args
+            self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_client_cert_and_key(self):
         action = 'https://host/action'
@@ -146,62 +159,58 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
         self.CONF.set_override("rest_notifier_certificate_file", certificate)
         self.CONF.set_override("rest_notifier_certificate_key", key)
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                poster.assert_called_with(action, data=mock.ANY,
-                                          headers=mock.ANY,
-                                          cert=(certificate, key), verify=True)
-                args, kwargs = poster.call_args
-                self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            poster.assert_called_with(action, data=mock.ANY,
+                                      headers=mock.ANY,
+                                      cert=(certificate, key), verify=True)
+            args, kwargs = poster.call_args
+            self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_verify_disable_by_cfg(self):
         action = 'https://host/action'
 
         self.CONF.set_override("rest_notifier_ssl_verify", False)
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                poster.assert_called_with(action, data=mock.ANY,
-                                          headers=mock.ANY,
-                                          verify=False)
-                args, kwargs = poster.call_args
-                self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            poster.assert_called_with(action, data=mock.ANY,
+                                      headers=mock.ANY,
+                                      verify=False)
+            args, kwargs = poster.call_args
+            self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_verify_disable(self):
         action = 'https://host/action?aodh-alarm-ssl-verify=0'
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                poster.assert_called_with(action, data=mock.ANY,
-                                          headers=mock.ANY,
-                                          verify=False)
-                args, kwargs = poster.call_args
-                self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            poster.assert_called_with(action, data=mock.ANY,
+                                      headers=mock.ANY,
+                                      verify=False)
+            args, kwargs = poster.call_args
+            self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_verify_enable_by_user(self):
         action = 'https://host/action?aodh-alarm-ssl-verify=1'
 
         self.CONF.set_override("rest_notifier_ssl_verify", False)
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                poster.assert_called_with(action, data=mock.ANY,
-                                          headers=mock.ANY,
-                                          verify=True)
-                args, kwargs = poster.call_args
-                self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            poster.assert_called_with(action, data=mock.ANY,
+                                      headers=mock.ANY,
+                                      verify=True)
+            args, kwargs = poster.call_args
+            self.assertEqual(self.HTTP_HEADERS, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
 
     @staticmethod
     def _fake_urlsplit(*args, **kwargs):
@@ -213,7 +222,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
             LOG = mock.MagicMock()
             with mock.patch('aodh.notifier.LOG', LOG):
                 self.service.notify_alarm(
-                    context.get_admin_context(),
+                    {},
                     {
                         'actions': ['no-such-action-i-am-sure'],
                         'alarm_id': 'foobar',
@@ -225,7 +234,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
         LOG = mock.MagicMock()
         with mock.patch('aodh.notifier.LOG', LOG):
             self.service.notify_alarm(
-                context.get_admin_context(),
+                {},
                 {
                     'actions': ['no-such-action-i-am-sure://'],
                     'alarm_id': 'foobar',
@@ -238,21 +247,61 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
         url = 'http://host/action'
 
         client = mock.MagicMock()
-        client.auth_token = 'token_1234'
+        client.session.auth.get_access.return_value.auth_token = 'token_1234'
         headers = {'X-Auth-Token': 'token_1234'}
         headers.update(self.HTTP_HEADERS)
 
         self.useFixture(mockpatch.Patch('keystoneclient.v3.client.Client',
                                         lambda **kwargs: client))
 
-        with mock.patch('eventlet.spawn_n', self._fake_spawn_n):
-            with mock.patch.object(requests.Session, 'post') as poster:
-                self.service.notify_alarm(context.get_admin_context(),
-                                          self._notification(action))
-                headers = {'X-Auth-Token': 'token_1234'}
-                headers.update(self.HTTP_HEADERS)
-                poster.assert_called_with(
-                    url, data=mock.ANY, headers=mock.ANY)
-                args, kwargs = poster.call_args
-                self.assertEqual(headers, kwargs['headers'])
-                self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+        with mock.patch.object(requests.Session, 'post') as poster:
+            self.service.notify_alarm({},
+                                      self._notification(action))
+            headers = {'X-Auth-Token': 'token_1234'}
+            headers.update(self.HTTP_HEADERS)
+            poster.assert_called_with(
+                url, data=mock.ANY, headers=mock.ANY)
+            args, kwargs = poster.call_args
+            self.assertEqual(headers, kwargs['headers'])
+            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+
+    def test_zaqar_notifier_action(self):
+        action = 'zaqar://?topic=critical&subscriber=http://example.com/data' \
+                 '&subscriber=mailto:foo@example.com&ttl=7200'
+        self.service.notify_alarm({},
+                                  self._notification(action))
+        self.assertEqual(self.zaqar,
+                         self.service.notifiers['zaqar'].obj.client)
+
+
+class FakeZaqarClient(object):
+
+    def __init__(self, testcase):
+        self.client = testcase
+
+    def queue(self, queue_name, **kwargs):
+        self.client.assertEqual('foobar-critical', queue_name)
+        self.client.assertEqual(dict(force_create=True), kwargs)
+        return FakeZaqarQueue(self.client)
+
+    def subscription(self, queue_name, **kwargs):
+        self.client.assertEqual('foobar-critical', queue_name)
+        subscribers = ['http://example.com/data', 'mailto:foo@example.com']
+        self.client.assertIn(kwargs['subscriber'], subscribers)
+        self.client.assertEqual('7200', kwargs['ttl'])
+
+
+class FakeZaqarQueue(object):
+
+    def __init__(self, testcase):
+        self.queue = testcase
+
+    def post(self, message):
+        expected_message = {'body': {'alarm_name': 'testalarm',
+                                     'reason_data': {'test': 'test'},
+                                     'current': 'ALARM',
+                                     'alarm_id': 'foobar',
+                                     'reason': 'what ?',
+                                     'severity': 'critical',
+                                     'previous': 'OK'}}
+        self.queue.assertEqual(expected_message, message)
